##
#  File:           RcsbLigandScoreProvider.py
#  Date:           10-Feb-2021 jdw
#
#  Update2:
#  25-Jul-2022 dwp  Change location of fall back files to point to master (not development) branch on py-rcsb_exdb_assets
##
"""
Accessors for RCSB Ligand quality score supporting data.

"""

import bisect
import logging
import math
import os.path
import statistics
import time

from rcsb.utils.io.FileUtil import FileUtil
from rcsb.utils.io.MarshalUtil import MarshalUtil
from rcsb.utils.io.StashableBase import StashableBase

logger = logging.getLogger(__name__)


class RcsbLigandScoreProvider(StashableBase):
    """Accessors for RCSB Ligand quality score supporting data."""

    def __init__(self, **kwargs):
        dirName = "rcsb-ligand-score"
        self.__cachePath = kwargs.get("cachePath", ".")
        super(RcsbLigandScoreProvider, self).__init__(self.__cachePath, [dirName])
        #
        self.__dirPath = os.path.join(self.__cachePath, dirName)
        self.__useCache = kwargs.get("useCache", True)
        rcsbLigandScoreUrl = kwargs.get("rcsbLigandScoreUrl", "https://github.com/rcsb/py-rcsb_exdb_assets/raw/master/fall_back/rcsb_ligand_score/ligand_score_reference.csv")
        rcsbLigandExcludeUrl = kwargs.get("rcsbLigandExcludeUrl", "https://github.com/rcsb/py-rcsb_exdb_assets/raw/master/fall_back/rcsb_ligand_score/ligand_score_exclude.list")
        #
        self.__mU = MarshalUtil(workPath=self.__dirPath)
        self.__ligandScoreDL, self.__ligandExcludeD = self.__reload(self.__dirPath, rcsbLigandScoreUrl, rcsbLigandExcludeUrl, self.__useCache)
        #
        self.__meanD = {}
        self.__stdD = {}
        self.__loadingD = {}
        self.__geoScoreList = None
        self.__fitScoreList = None

    def testCache(self):
        if self.__ligandScoreDL and self.__ligandExcludeD:
            logger.info("Ligand score (%d) exclude (%d)", len(self.__ligandScoreDL), len(self.__ligandExcludeD))
            return True
        return False

    def getLigandExcludeList(self):
        return list(self.__ligandExcludeD.keys())

    def isLigandExcluded(self, ccId):
        return ccId in self.__ligandExcludeD

    def getFitScoreRanking(self, score):
        try:
            if not self.__fitScoreList:
                self.__fitScoreList = sorted([float(tD["fit_pc1"]) for tD in self.__ligandScoreDL])
                logger.debug("Sorted model fit score (%d) range %.3f : %.3f", len(self.__fitScoreList), self.__fitScoreList[0], self.__fitScoreList[-1])
                logger.debug("Sorted model fit score (%d) range %.3f : %.3f", len(self.__fitScoreList), min(self.__fitScoreList), max(self.__fitScoreList))
            frac = bisect.bisect(self.__fitScoreList, score) / float(len(self.__fitScoreList) - 1)
            return 1.0 - frac
            #
        except Exception as e:
            logger.exception("Failing with %s", str(e))
        return 0

    def getGeometryScoreRanking(self, score):
        try:
            if not self.__geoScoreList:
                self.__geoScoreList = sorted([float(tD["geo_pc1"]) for tD in self.__ligandScoreDL])
                logger.debug("Sorted model geometry score (%d) range %.3f : %.3f", len(self.__geoScoreList), self.__geoScoreList[0], self.__geoScoreList[-1])
                logger.debug("Sorted model geometry score (%d) range %.3f : %.3f", len(self.__geoScoreList), min(self.__geoScoreList), max(self.__geoScoreList))
            frac = bisect.bisect(self.__geoScoreList, score) / float(len(self.__geoScoreList) - 1)
            return 1.0 - frac
        except Exception as e:
            logger.exception("Failing with %s", str(e))
        return 0

    def __reload(self, dirPath, rcsbLigandScoreUrl, rcsbLigandExcludeUrl, useCache):
        startTime = time.time()
        ligandScoreDL = []
        ligandExcludeD = {}
        #
        ok = False
        fU = FileUtil()
        fU.mkdir(dirPath)
        #
        fn = os.path.basename(rcsbLigandScoreUrl)
        ligandScoreFilePath = os.path.join(dirPath, fn)
        fn = os.path.basename(rcsbLigandExcludeUrl)
        ligandExcludeFilePath = os.path.join(dirPath, fn)
        #
        if useCache and fU.exists(ligandScoreFilePath) and fU.exists(ligandExcludeFilePath):
            ok = True
        elif not useCache:
            logger.info("Fetching url %s path %s", rcsbLigandScoreUrl, ligandScoreFilePath)
            ok = fU.get(rcsbLigandScoreUrl, ligandScoreFilePath)
            logger.info("Fetching url %s path %s", rcsbLigandExcludeUrl, ligandExcludeFilePath)
            ok = fU.get(rcsbLigandExcludeUrl, ligandExcludeFilePath)
            logger.info("Completed fetch (%r) at %s (%.4f seconds)", ok, time.strftime("%Y %m %d %H:%M:%S", time.localtime()), time.time() - startTime)
            #
        if ok:
            ligandScoreDL = self.__mU.doImport(ligandScoreFilePath, fmt="csv", rowFormat="dict")
            ligExcludeL = self.__mU.doImport(ligandExcludeFilePath, fmt="list")
            ligandExcludeD = {lig: True for lig in ligExcludeL}
            # ---
        return ligandScoreDL, ligandExcludeD

    def getParameterStatistics(self):
        """Return the mean, standard deviation and paramter loadings for score model parameters

        Returns:
            (dict,dict,dict): mean, std. dev., and loading ({"rsr": v, "rscc": v, "mogul_bonds_rmsz": v, "mogul_angles_rmsz": v})
        """
        if not (self.__meanD and self.__stdD and self.__loadingD):
            self.__meanD, self.__stdD, self.__loadingD = self.__calcParameterStatistics()
        return self.__meanD, self.__stdD, self.__loadingD

    def __calcParameterStatistics(self):
        """Calculatefe the mean, standard deviation and paramter loadings for score model parameters

        Returns:
            (dict,dict,dict): mean, std. dev., and loading ({"rsr": v, "rscc": v, "mogul_bonds_rmsz": v, "mogul_angles_rmsz": v})
        """
        meanD = {}
        stdD = {}
        loadingD = {}
        try:
            #
            for ky in ["rsr", "rscc", "mogul_bonds_rmsz", "mogul_angles_rmsz"]:
                tL = [float(tD[ky]) for tD in self.__ligandScoreDL]
                meanD[ky] = statistics.mean(tL)
                stdD[ky] = statistics.stdev(tL)
                loadingD[ky] = math.sqrt(2.0) / 2.0 if ky != "rscc" else -math.sqrt(2.0) / 2.0
                #
        except Exception as e:
            logger.exception("Failing with %s", str(e))

        return meanD, stdD, loadingD



##
# File:    ResidProviderTests.py
# Author:  J. Westbrook
# Date:    18-Mar-2020
# Version: 0.001
#
# Update:
#
#
##
"""
Test utilities to read the RESID resource file and build loadable documents and identifier
    correspondences.

"""

__docformat__ = "google en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2.0"

import logging
import os
import unittest

from rcsb.utils.chemref.ResidProvider import ResidProvider

HERE = os.path.abspath(os.path.dirname(__file__))
TOPDIR = os.path.dirname(os.path.dirname(HERE))

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


class ResidProviderTests(unittest.TestCase):
    def setUp(self):
        self.__workPath = os.path.join(HERE, "test-output")

    def tearDown(self):
        pass

    def testReadResidData(self):
        residP = ResidProvider(cachePath=self.__workPath, useCache=False)
        ok = residP.testCache()
        self.assertTrue(ok)
        version = residP.getVersion()
        self.assertTrue(version is not None)
        #


def readResidData():
    suiteSelect = unittest.TestSuite()
    suiteSelect.addTest(ResidProviderTests("testReadResidData"))
    return suiteSelect


if __name__ == "__main__":

    mySuite = readResidData()
    unittest.TextTestRunner(verbosity=2).run(mySuite)



##
#  File:           CODProvider.py
#  Date:           8-Feb-2021 jdw
#
#  Updated:
#
##
"""
Accessors for Crystallographic Open Database (COD) molecule data.

"""

import logging
import os.path
import time

from rcsb.utils.io.FileUtil import FileUtil
from rcsb.utils.io.MarshalUtil import MarshalUtil

logger = logging.getLogger(__name__)


class CODProvider:
    """Accessors for COD molecule data."""

    def __init__(self, **kwargs):
        #
        self.__cachePath = kwargs.get("cachePath", ".")
        self.__dirPath = os.path.join(self.__cachePath, "COD-molecules")
        self.__codCifDirPath = os.path.join(self.__dirPath, "CIF")
        self.__useCache = kwargs.get("useCache", True)
        codSmilesDumpUrl = kwargs.get("CODSmilesDumpUrl", "http://www.crystallography.net/cod/smi/allcod.smi")
        #
        self.__codCifTemplateUrl = "http://www.crystallography.net/cod/"
        self.__mU = MarshalUtil(workPath=self.__dirPath)
        ok = self.__reload(self.__dirPath, codSmilesDumpUrl, self.__useCache)
        #
        logger.info("COD SMILES data status (%r)", ok)
        #

    def testCache(self, minCount=200000):
        _ = minCount
        return True

    def getSmilesPath(self):
        return os.path.join(self.__dirPath, "cod-molecules.smi")

    def __reload(self, dirPath, codSmilesDumpUrl, useCache):
        startTime = time.time()

        ok = False
        fU = FileUtil()
        codSmilesDumpFileName = "allcod.smi"
        codSmilesDumpPath = os.path.join(dirPath, codSmilesDumpFileName)
        #
        fU.mkdir(dirPath)
        codSmilesDataPath = self.getSmilesPath()
        #
        logger.info("useCache %r CODSmilesDumpPath %r", useCache, codSmilesDumpPath)
        if useCache and fU.exists(codSmilesDataPath):
            #
            ok = True
        else:
            logger.info("Fetching url %s path %s", codSmilesDumpUrl, codSmilesDumpPath)
            ok = fU.get(codSmilesDumpUrl, codSmilesDumpPath)
            logger.info("Completed fetch (%r) at %s (%.4f seconds)", ok, time.strftime("%Y %m %d %H:%M:%S", time.localtime()), time.time() - startTime)
            #
            numSmiles = self.__reformatCodSmilesData(codSmilesDumpPath, codSmilesDataPath)
            ok = numSmiles > 200000
        # ---
        return ok

    def __reformatCodSmilesData(self, inpFilePath, outFilePath):
        """Reformat COD SMILES data -

        Args:
            inpFilePath (str): input file path for COD molecule data
            outFilePath (str): reformatted file path for COD molecule data

        Returns:
            (int):  number of reformatted SMILES records
        """
        sCount = 0
        try:
            nl = "\n"
            with open(inpFilePath, "r", encoding="utf-8") as ifh, open(outFilePath, "w", encoding="utf-8") as ofh:
                for line in ifh.readlines():
                    ff = line[:-1].split("\t")
                    ff.reverse()
                    ofh.write("\t".join(ff) + nl)
                    sCount += 1
            #
            logger.info("Parsed COD molecule references (%d)", sCount)
        except Exception as e:
            logger.exception("Failing using %r with %s", inpFilePath, str(e))
        return sCount

    def getCodSmilesList(self):
        smiTupL = []
        inpFilePath = self.getSmilesPath()
        try:
            with open(inpFilePath, "r", encoding="utf-8") as ifh:
                for line in ifh.readlines():
                    ff = line[:-1].split("\t")
                    smiTupL.append((ff[0], ff[1]))
            return smiTupL
        except Exception as e:
            logger.exception("Failing for %r with %s", inpFilePath, str(e))

    def getCifPath(self, codId):
        return os.path.join(self.__codCifDirPath, codId[-1], codId + ".cif")

    def fetchCif(self, codId):
        ok = False
        try:
            startTime = time.time()
            fU = FileUtil()
            fU.mkdir(self.__codCifDirPath)
            codCifPath = self.getCifPath(codId)
            #
            logger.info("useCache %r codCifPath %r", self.__useCache, codCifPath)
            if self.__useCache and fU.exists(codCifPath):
                ok = True
            else:
                codCifUrl = os.path.join(self.__codCifTemplateUrl, codId + ".cif")
                logger.info("Fetching url %s path %s", codCifUrl, codCifPath)
                ok = fU.get(codCifUrl, codCifPath)
                logger.info("Completed fetch (%r) at %s (%.4f seconds)", ok, time.strftime("%Y %m %d %H:%M:%S", time.localtime()), time.time() - startTime)
        except Exception as e:
            logger.exception("Failing for %r with %s", codId, str(e))
            # ---
        return ok



##
# File:    ChemCompModelProviderTests.py
# Author:  J. Westbrook
# Date:    1-Nov-2018
# Version: 0.001
#
# Update:
#
#
##
"""
Various utilities for extracting data from CCDC correspondence data files.

"""

__docformat__ = "google en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2.0"

import logging
import os
import unittest

from rcsb.utils.chemref.ChemCompModelProvider import ChemCompModelProvider

HERE = os.path.abspath(os.path.dirname(__file__))
TOPDIR = os.path.dirname(os.path.dirname(HERE))

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


class ChemCompModelProviderTests(unittest.TestCase):
    def setUp(self):
        self.__workPath = os.path.join(HERE, "test-output")

    def tearDown(self):
        pass

    def testReadChemCompModelRef(self):
        ccm = ChemCompModelProvider(cachePath=self.__workPath, useCache=False)
        ok = ccm.testCache()
        self.assertTrue(ok)
        rD = ccm.getMapping()
        self.assertGreater(len(rD), 1000)
        aD = ccm.getMapping()
        self.assertGreater(len(aD), 1000)
        #
        #
        logger.info("Model match length %d", len(rD))
        for ccId in rD:
            if len(rD[ccId]) > 1:
                logger.debug("Match length for %r %d", ccId, len(rD[ccId]))


def readChemCompModelInfo():
    suiteSelect = unittest.TestSuite()
    suiteSelect.addTest(ChemCompModelProviderTests("testReadChemCompModelRef"))
    return suiteSelect


if __name__ == "__main__":

    mySuite = readChemCompModelInfo()
    unittest.TextTestRunner(verbosity=2).run(mySuite)



##
# File:    PharosProviderTests.py
# Author:  J. Westbrook
# Date:    27-Jul-2018
#
# Update:
#
#
##
"""
Tests for accessors for managing PubChem extracted annotations.

"""

__docformat__ = "google en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2.0"

import logging
import os
import unittest

from rcsb.utils.chemref.PharosProvider import PharosProvider
from rcsb.utils.io.MarshalUtil import MarshalUtil

HERE = os.path.abspath(os.path.dirname(__file__))
TOPDIR = os.path.dirname(os.path.dirname(HERE))

logging.basicConfig(level=logging.INFO, format="%(asctime)s [%(levelname)s]-%(module)s.%(funcName)s: %(message)s")
logger = logging.getLogger()


class PharosProviderTests(unittest.TestCase):
    def setUp(self):
        self.__cachePath = os.path.join(HERE, "test-output")
        self.__dataPath = os.path.join(HERE, "test-data")
        self.__mU = MarshalUtil(workPath=self.__cachePath)
        self.__phL = self.__mU.doImport(os.path.join(self.__dataPath, "Pharos-chembl-id-20200730.list"), fmt="list")

    def tearDown(self):
        pass

    def testPharosAnnotBootstrap(self):
        minCount = 266600
        pcP = PharosProvider(cachePath=self.__cachePath, useCache=False)
        ok = pcP.load(self.__phL, "identifiers", fmt="json", indent=3)
        self.assertTrue(ok)
        riD = pcP.getIdentifiers()
        logger.info("riD (%d)", len(riD))
        ok = pcP.testCache(minCount=minCount)
        #
        pcP = PharosProvider(cachePath=self.__cachePath, useCache=True)
        ok = pcP.testCache(minCount=minCount)
        self.assertTrue(ok)


def pharosAnnotSuite():
    suiteSelect = unittest.TestSuite()
    suiteSelect.addTest(PharosProviderTests("testPharosAnnotBootstrap"))
    return suiteSelect


if __name__ == "__main__":
    mySuite = pharosAnnotSuite()
    unittest.TextTestRunner(verbosity=2).run(mySuite)



##
# File:    AtcProviderTests.py
# Author:  J. Westbrook
# Date:    1-Nov-2018
# Version: 0.001
#
# Update:
#
#
##
"""
Various utilities for extracting data from the ATC hierarchy.

"""

__docformat__ = "google en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2.0"

import logging
import os
import unittest

from rcsb.utils.chemref.AtcProvider import AtcProvider

HERE = os.path.abspath(os.path.dirname(__file__))
TOPDIR = os.path.dirname(os.path.dirname(HERE))

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


class AtcProviderTests(unittest.TestCase):
    def setUp(self):
        self.__cachePath = os.path.join(HERE, "test-output")

    def tearDown(self):
        pass

    def testReadAtcInfo(self):
        atcP = AtcProvider(cachePath=self.__cachePath, useCache=False)
        version = atcP.getVersion()
        self.assertTrue(version is not None)
        tnL = atcP.getTreeNodeList()
        logger.info("length of tree list %d", len(tnL))
        self.assertGreaterEqual(len(tnL), 6000)
        idLin = atcP.getIdLineage("B05DB")
        logger.info("ID Lineage %r", idLin)
        atcName = atcP.getAtcName("B05DB")
        logger.info("ATC Name %r", atcName)
        nameLin = atcP.getNameLineage("B05DB")
        logger.info("Name Lineage %r", nameLin)
        # for tn in tnL:
        #    logger.info(">>>node %r", tn)


def readAtcInfo():
    suiteSelect = unittest.TestSuite()
    suiteSelect.addTest(AtcProviderTests("testReadAtcInfo"))
    return suiteSelect


if __name__ == "__main__":

    mySuite = readAtcInfo()
    unittest.TextTestRunner(verbosity=2).run(mySuite)



##
# File:    ChemCompProviderTests.py
# Author:  J. Westbrook
# Date:    1-Nov-2018
# Version: 0.001
#
# Updates:
# 28-Mar-2022 bv Fix pylint issue with "Iterated dict modified inside for loop body" in testChemCompProvider
#  7-Jun-2022 aae Include bond count in chem comp data and fix typo
#
##
"""
Various utilities for extracting data from chemical component definitions.

"""

__docformat__ = "google en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2.0"

import datetime
import logging
import os
import unittest
import copy

from rcsb.utils.chemref.ChemCompProvider import ChemCompProvider
from rcsb.utils.io.MarshalUtil import MarshalUtil
from rcsb.utils.io.TimeUtil import TimeUtil


HERE = os.path.abspath(os.path.dirname(__file__))
TOPDIR = os.path.dirname(os.path.dirname(HERE))

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


class ChemCompProviderTests(unittest.TestCase):
    def setUp(self):
        self.__workPath = os.path.join(HERE, "test-output")
        self.__dataPath = os.path.join(HERE, "test-data")

    def tearDown(self):
        pass

    @unittest.skip("maintenance test")
    def testProcessReleaseData(self):
        """Test process raw chemical component release data extracted from released and obsolete entries"""
        try:
            fpOut = os.path.join(self.__workPath, "chem-comp-release-data-summary.json")
            fpRel = os.path.join(self.__dataPath, "chem-comp-release-data.json.gz")
            fpObs = os.path.join(self.__dataPath, "pdbx_obsolete-chem-comp-release-data.json")
            tU = TimeUtil()
            mU = MarshalUtil(workPath=self.__workPath)
            relTupL = mU.doImport(fpRel, fmt="json")
            obsTupL = mU.doImport(fpObs, fmt="json")
            logger.info("rel length (%d) obs length (%d)", len(relTupL), len(obsTupL))
            cD = {}
            for entryId, tS, ccIdL in obsTupL + relTupL:
                if len(tS) < 4:
                    continue
                dtObj = tU.getDateTimeObj(tS)
                for ccId in ccIdL:
                    if ccId not in cD:
                        cD[ccId] = (entryId, dtObj)
                    elif dtObj < cD[ccId][1]:
                        cD[ccId] = (entryId, dtObj)
            #
            logger.info("ALA %r %s", cD["ALA"][0], cD["ALA"][1])
            logger.info("cD (%d)", len(cD))
            cloneD = copy.deepcopy(cD)
            for ccId in cloneD:
                cD[ccId] = (cloneD[ccId][0], cloneD[ccId][1].strftime("%Y-%m-%d"))

            tS = datetime.datetime.now().isoformat()
            vS = datetime.datetime.now().strftime("%Y-%m-%d")
            ok = mU.doExport(fpOut, {"version": vS, "created": tS, "release_data": cD}, fmt="json", indent=3)
            self.assertTrue(ok)
        except Exception as e:
            logger.exception("Failing with %s", str(e))

    def testReadChemCompRef(self):
        ccP = ChemCompProvider(cachePath=self.__workPath, useCache=False)
        ok = ccP.testCache()
        self.assertTrue(ok)
        #
        rD = ccP.getAbridged()
        self.assertGreaterEqual(len(rD), 30000)
        #
        nsL = []
        for ccId in ccP.getComponentIds():
            tS = ccP.getParentComponent(ccId)
            if tS:
                nsL.append(tS)
        logger.info("nsL %r", len(nsL))
        self.assertGreaterEqual(len(nsL), 1550)
        #
        ccP = ChemCompProvider(cachePath=self.__workPath, useCache=True)
        ok = ccP.testCache()
        self.assertTrue(ok)
        numAtoms = ccP.getAtomCount("ATP")
        numChiralAtoms = ccP.getAtomCountChiral("ATP")
        numHeavyAtoms = ccP.getAtomCountHeavy("ATP")
        numBonds = ccP.getBondCount("ATP")
        logger.debug("%r %r %r %r", numAtoms, numChiralAtoms, numHeavyAtoms, numBonds)
        self.assertEqual(numAtoms, 47)
        self.assertEqual(numChiralAtoms, 6)
        self.assertEqual(numHeavyAtoms, 31)
        self.assertEqual(numBonds, 49)

        ccIdL = ccP.getComponentIds()
        iCount = 0
        fwCount = 0
        for ccId in ccIdL:
            if ccP.getAtomCountHeavy(ccId) < 1:
                logger.info("%s has no heavy atoms", ccId)
                iCount += 1
            if not ccP.getFormulaWeight(ccId):
                logger.info("%s has no formula weight", ccId)
                fwCount += 1
        self.assertGreaterEqual(3, iCount)
        self.assertGreaterEqual(3, fwCount)
        #
        tS = ccP.getReleaseDate("ALA")
        self.assertEqual(tS, "1973-05-03")
        fEntryId = ccP.getEntryOfFirstRelease("ALA")
        self.assertEqual(fEntryId, "2pti")


def readChemCompInfo():
    suiteSelect = unittest.TestSuite()
    suiteSelect.addTest(ChemCompProviderTests("testReadChemCompRef"))
    return suiteSelect


if __name__ == "__main__":

    mySuite = readChemCompInfo()
    unittest.TextTestRunner(verbosity=2).run(mySuite)



##
#  File:           AtcProvider.py
#  Date:           3-Apr-2019 jdw
#
#  Updates:
#   5-Jun-2021 jdw  Update ATC data source and fallback
#  20-Jul-2021 jdw  Make this provider a subclass of StashableBase
#   3-Jan-2022 dwp  Update data loading methods to address recent changes in source NCBO ATC files
#  25-Jul-2022 dwp  Revert last change - source NCBO ATC files were updated again to restore previous format
#  27-Jul-2022 dwp  Adapt to possible naming schemes for class IDs (may be identified with "ATC" or "UATC")
#  22-Oct-2024 dwp  Add standardization method for ATC data to handle potential stringified arrays
##
"""
  Extract ATC term descriptions from NCBO ATC flat files.

"""

import collections
import logging
import os.path
import sys
import ast

from rcsb.utils.io.FileUtil import FileUtil
from rcsb.utils.io.MarshalUtil import MarshalUtil
from rcsb.utils.io.StashableBase import StashableBase


logger = logging.getLogger(__name__)


class AtcProvider(StashableBase):
    """Extract term descriptions and ATC classifications from ATC flat files."""

    def __init__(self, **kwargs):
        self.__dirName = "atc"
        self.__cachePath = kwargs.get("cachePath", ".")
        super(AtcProvider, self).__init__(self.__cachePath, [self.__dirName])
        atcDirPath = os.path.join(self.__cachePath, self.__dirName)
        useCache = kwargs.get("useCache", True)

        urlTarget = "https://data.bioontology.org/ontologies/ATC/download?apikey=8b5b7825-538d-40e0-9e9e-5ab9274a9aeb&download_format=csv"
        urlTargetFallback = "https://github.com/rcsb/py-rcsb_exdb_assets/raw/master/fall_back/ATC.csv.gz"
        self.__version = kwargs.get("AtcVersion", "2023")
        #
        self.__mU = MarshalUtil(workPath=atcDirPath)
        self.__atcD = self.__reload(urlTarget, urlTargetFallback, atcDirPath, useCache=useCache)
        #

    def getVersion(self):
        return self.__version

    def __reload(self, urlTarget, urlTargetFallback, atcDirPath, useCache=True):
        pyVersion = sys.version_info[0]
        atcFilePath = os.path.join(atcDirPath, "atc-py%s.pic" % str(pyVersion))
        #
        atcD = {}
        if useCache and self.__mU.exists(atcFilePath):
            atcD = self.__mU.doImport(atcFilePath, fmt="pickle")
            # logger.debug("ATC name length %d parent length %d assignments %d", len(atcD["names"]), len(atcD["parents"]), len(atcD["assignments"]))
            # nD = atcD["names"]
            # pD = atcD["parents"]

        elif not useCache:
            fn = "ATC.csv.gz"
            fp = os.path.join(atcDirPath, fn)
            logger.debug("Fetch ATC term descriptions from source %s", fp)
            fileU = FileUtil(workPath=atcDirPath)
            fileU.mkdir(atcDirPath)
            try:
                ok = fileU.get(urlTarget, fp)
                logger.info("ATC fetch status is %r", ok)
                if not ok:
                    ok = fileU.get(urlTargetFallback, fp)
                    logger.info("ATC fallback fetch status is %r", ok)
                atcL = self.__mU.doImport(fp, fmt="csv", rowFormat="dict", uncomment=False)
                #
                columnL = list(atcL[0].keys()) if atcL else []
                nD = self.__extractNames(atcL)
                pD = self.__extractHierarchy(atcL)
                atcD = {"names": nD, "parents": pD}
                ok = self.__mU.doExport(atcFilePath, atcD, fmt="pickle")
                logger.info("ATC cache status %r data length %d columns %r names %d parents %d", ok, len(atcL), columnL, len(nD), len(pD))
            except Exception as e:
                logger.exception("Failing for %r with %s", fp, str(e))
            #
        return atcD

    def getAtcVersion(self):
        return self.__version

    def testCache(self):
        if self.__atcD and "names" in self.__atcD and "parents" in self.__atcD and (len(self.__atcD["names"]) > 6100) and (len(self.__atcD["parents"]) > 6100):
            logger.info("ATC names %d parents %d", len(self.__atcD["names"]), len(self.__atcD["parents"]))
            return True
        return False

    def getAtcName(self, atcId):
        try:
            return self.__atcD["names"][atcId]["name"]
        except Exception:
            logger.info("Undefined ATC  %r", atcId)
        return None

    def getIdLineage(self, atcId):
        pList = []
        try:
            pList.append(atcId)
            pt = self.__atcD["parents"][atcId]
            while (pt is not None) and (pt != 0):
                pList.append(pt)
                pt = self.__atcD["parents"][pt]
        except Exception as e:
            logger.exception("Failing for %r with %s", atcId, str(e))
        #
        pList.reverse()
        return pList

    def getNameLineage(self, atcId):
        try:
            return [self.getAtcName(cId) for cId in self.getIdLineage(atcId)]
        except Exception as e:
            logger.exception("Failing for %r with %s", atcId, str(e))
        return None

    def getTreeNodeList(self, filterD=None):
        return self.__exportTreeNodeList(self.__atcD["names"], self.__atcD["parents"], filterD=filterD)

    def __extractNames(self, atcL):
        """
        Example public data set from BioPortal -
        Class ID,Preferred Label,Synonyms,Definitions,Obsolete,CUI,Semantic Types,Parents,ATC LEVEL,Is Drug Class,Semantic type UMLS property

        http://purl.bioontology.org/ontology/ATC/A03AX13,silicones,,,false,C0037114,http://purl.bioontology.org/ontology/STY/T109|http://purl.bioontology.org/ontology/STY/T122,http://purl.bioontology.org/ontology/UATC/A03AX,5,,http://purl.bioontology.org/ontology/STY/T109|http://purl.bioontology.org/ontology/STY/T122
        http://purl.bioontology.org/ontology/ATC/J01DB07,cefatrizine,,,false,C0007545,http://purl.bioontology.org/ontology/STY/T195|http://purl.bioontology.org/ontology/STY/T109,http://purl.bioontology.org/ontology/UATC/J01DB,5,,http://purl.bioontology.org/ontology/STY/T195|http://purl.bioontology.org/ontology/STY/T109

        Occasionally changes from "ATC" to "UATC", so capture these too:
        http://purl.bioontology.org/ontology/UATC/A03AX13,silicones,,,false,C0037114,http://purl.bioontology.org/ontology/STY/T109|http://purl.bioontology.org/ontology/STY/T122,http://purl.bioontology.org/ontology/UATC/A03AX,5,,http://purl.bioontology.org/ontology/STY/T109|http://purl.bioontology.org/ontology/STY/T122
        http://purl.bioontology.org/ontology/UATC/J01DB07,cefatrizine,,,false,C0007545,http://purl.bioontology.org/ontology/STY/T195|http://purl.bioontology.org/ontology/STY/T109,http://purl.bioontology.org/ontology/UATC/J01DB,5,,http://purl.bioontology.org/ontology/STY/T195|http://purl.bioontology.org/ontology/STY/T109
        """
        nD = {}
        nsATC = "http://purl.bioontology.org/ontology/ATC/"
        nsUATC = "http://purl.bioontology.org/ontology/UATC/"
        for rD in atcL:
            if nsATC in rD["Class ID"]:
                ns = nsATC
            elif nsUATC in rD["Class ID"]:
                ns = nsUATC
            else:
                continue
            idCode = rD["Class ID"].replace(ns, "")
            name = self.__standardizePreferredLabel(rD["Preferred Label"])
            synonyms = rD["Synonyms"]
            definition = rD["Definitions"]
            level = int(rD["ATC LEVEL"]) if rD["ATC LEVEL"].isdigit() else None
            nD[idCode] = {"name": name, "synonyms": synonyms, "definition": definition, "level": level}
        logger.info("Length of name dictionary %d", len(nD))
        return nD

    def __standardizePreferredLabel(self, label):
        """Temporary method to standardize the "Preferred Label" value of an ATC datum,
        due to recent unintended changes to source ATC data in which the column is appearing
        as a stringified array instead of a simple string. E.g.,:

        Before:
            http://purl.bioontology.org/ontology/ATC/B05DB,Hypertonic solutions,...<remainder_omitted>

        Now (as of Version 2024AA):
            http://purl.bioontology.org/ontology/ATC/B05DB,"[""Hypertonic solutions""]",...<remainder_omitted>

        Once this is resolved at the source data level, can stop using this method.

        Args:
            label (str): raw label

        Returns:
            label: processed label
        """
        try:
            # Attempt to parse the label as a list
            parsedLabel = ast.literal_eval(label)
            # If it's a list, return the first element (assuming it's a single-item array)
            if isinstance(parsedLabel, list) and len(parsedLabel) == 1:
                return parsedLabel[0]
            else:
                logger.info("parsed label %r", parsedLabel)
        except (ValueError, SyntaxError):
            # If not a list or parsing fails, return the original label
            return label
        return label

    def __extractHierarchy(self, atcL):
        """ """
        pD = {}
        nsATC = "http://purl.bioontology.org/ontology/ATC/"
        nsUATC = "http://purl.bioontology.org/ontology/UATC/"
        for rD in atcL:
            if nsATC in rD["Class ID"]:
                ns = nsATC
            elif nsUATC in rD["Class ID"]:
                ns = nsUATC
            else:
                continue
            idCode = rD["Class ID"].replace(ns, "")
            pIdCode = rD["Parents"].replace(ns, "")
            pIdCode = None if "http://www.w3.org/2002/07/owl#Thing" in pIdCode else pIdCode
            pD[idCode] = pIdCode
        #
        logger.info("Length of parent dictionary %d", len(pD))
        # for ky, vl in pD.items():
        #    logger.info(" %r  %r", ky, vl)
        return pD

    def __exportTreeNodeList(self, nD, pD, filterD=None):
        """Create node list from the ATC  parent and name/description dictionaries."""
        #
        # pL = [0]
        pL = []
        logger.info("nD %d pD %d", len(nD), len(pD))
        # create child dictionary
        cD = {}
        for ctId, ptId in pD.items():
            if not ptId:
                pL.append(ctId)
                continue
            cD.setdefault(ptId, []).append(ctId)

        #
        logger.debug("cD %d", len(cD))
        #
        idL = []
        for rootId in sorted(pL):
            visited = set([rootId])
            queue = collections.deque(visited)
            while queue:
                tId = queue.popleft()
                idL.append(tId)
                if tId not in cD:
                    # logger.warning("No children for Atc tId %r" % tId)
                    continue
                for childId in cD[tId]:
                    if childId not in visited:
                        queue.append(childId)
                        visited.add(childId)
        #
        uD = {}
        dL = []
        for tId in idL:
            if filterD and tId not in filterD:
                continue
            # id plus parents
            idlL = self.getIdLineage(tId)
            for iDepth, idl in enumerate(idlL, 0):
                if idl in uD:
                    continue
                uD[idl] = True
                displayName = nD[idl]["name"] if idl in nD else None
                ptId = pD[idl] if idl in pD else None
                #
                # dD = {"id": tId, "name": displayName, "lineage": lL, "parents": [ptId], "depth": len(lL)}
                #
                if ptId:
                    dD = {"id": idl, "name": displayName, "parents": [ptId], "depth": iDepth}
                else:
                    dD = {"id": idl, "name": displayName, "depth": iDepth}
                #
                dL.append(dD)

        return dL



##
# File:    PubChemUtilsTests.py
# Author:  J. Westbrook
# Date:    1-May-2020
# Version: 0.001
#
# Update:
#
#
##
"""
Test access to PubChem fetch and search utilities.

"""

__docformat__ = "google en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2.0"

import logging
import os
import unittest

from rcsb.utils.chemref.PubChemUtils import PubChemUtils, ChemicalIdentifier

HERE = os.path.abspath(os.path.dirname(__file__))
TOPDIR = os.path.dirname(os.path.dirname(HERE))

logging.basicConfig(level=logging.INFO, format="%(asctime)s [%(levelname)s]-%(module)s.%(funcName)s: %(message)s")
logger = logging.getLogger()


class PubChemUtilsTests(unittest.TestCase):
    def setUp(self):
        self.__workPath = os.path.join(HERE, "test-output", "PubChem")

    def tearDown(self):
        pass

    def testFetchCompoundRecord(self):
        cId = "2244"
        cName = "2-acetyloxybenzoic acid"
        #
        pcU = PubChemUtils()
        chemId = ChemicalIdentifier(idCode="aspirin|test", identifier=cId, identifierType="cid")
        rawPath = os.path.join(self.__workPath, "%s-pubchem-record-raw.json" % cId)
        extractedPath = os.path.join(self.__workPath, "%s-pubchem-record-extracted.json" % cId)
        retStatus, rDL = pcU.fetch(chemId, returnType="record", storeRawResponsePath=rawPath, storeResponsePath=extractedPath)
        self.assertTrue(retStatus)
        ok = self.__containsCid(rDL, cId)
        self.assertTrue(ok)
        #
        chemId = ChemicalIdentifier(idCode="aspirin|test", identifier=cName, identifierType="name")
        retStatus, rDL = pcU.fetch(chemId, returnType="record")
        self.assertTrue(retStatus)
        ok = self.__containsCid(rDL, cId)
        self.assertTrue(ok)
        #

    def __containsCid(self, rDL, cid):
        try:
            for rD in rDL:
                if "cid" in rD and rD["cid"] == cid:
                    return True
            return False
        except Exception:
            pass
        return False

    def testSearchCompoundRecord(self):
        pcU = PubChemUtils()
        cId = "2244"
        chemId = ChemicalIdentifier(idCode="aspirin|test", identifier="CC(=O)OC1=CC=CC=C1C(=O)O", identifierType="smiles", identifierSource="3d-model")
        retStatus, rDL = pcU.fetch(chemId, searchType="fastidentity", returnType="record")
        self.assertTrue(retStatus)
        ok = self.__containsCid(rDL, cId)
        self.assertTrue(ok)
        #
        chemId = ChemicalIdentifier(
            idCode="aspirin|test", identifier="InChI=1S/C9H8O4/c1-6(10)13-8-5-3-2-4-7(8)9(11)12/h2-5H,1H3,(H,11,12)", identifierType="inchi", identifierSource="3d-model"
        )
        retStatus, rDL = pcU.fetch(chemId, searchType="fastidentity")
        self.assertTrue(retStatus)
        ok = self.__containsCid(rDL, cId)
        self.assertTrue(ok)
        #
        chemId = ChemicalIdentifier(idCode="aspirin|test", identifier="BSYNRYMUTXBXSQ-UHFFFAOYSA-N", identifierType="inchikey", identifierSource="3d-model")
        retStatus, rDL = pcU.fetch(chemId, searchType="lookup", returnType="record")
        self.assertTrue(retStatus)
        ok = self.__containsCid(rDL, cId)
        self.assertTrue(ok)
        #
        chemId = ChemicalIdentifier(idCode="Dummy", identifier="BSYNRYMUTXBXSQ-UHFFFAOYSA-Q", identifierType="inchikey", identifierSource="3d-model")
        retStatus, rDL = pcU.fetch(chemId, searchType="lookup", returnType="record")
        self.assertFalse(retStatus)
        self.assertFalse(rDL)

    # @unittest.skip("Skipping until api is more reliable")
    def testFetchCompoundAltReturnTypes(self):
        pcU = PubChemUtils()
        cIdList = ["123631", "2244"]
        for cId in cIdList:
            chemId = ChemicalIdentifier(idCode=cId, identifier=cId, identifierType="cid")
            for returnType, _ in [("classification", "Hierarchies"), ("property", "PropertyTable"), ("xrefs", "InformationList"), ("synonyms", "InformationList")]:
                rawResponsePath = os.path.join(self.__workPath, "%s-pubchem-%s-raw.json" % (cId, returnType))
                extractedResponsePath = os.path.join(self.__workPath, "%s-pubchem-%s-extract.json" % (cId, returnType))
                retStatus, rDL = pcU.fetch(chemId, returnType=returnType, storeRawResponsePath=rawResponsePath, storeResponsePath=extractedResponsePath)
                self.assertTrue(retStatus)
                ok = self.__containsCid(rDL, cId)
                self.assertTrue(ok)

    def testFetchCompoundView(self):
        cIdList = ["2244", "123631"]
        for cId in cIdList:
            pcU = PubChemUtils()
            chemId = ChemicalIdentifier(idCode="test", identifierType="cid", identifier=cId)
            rawResponsePath = os.path.join(self.__workPath, "%s-pubchem-view-raw.json" % cId)
            extractedResponsePath = os.path.join(self.__workPath, "%s-pubchem-view-extracted.json" % cId)
            retStatus, vL = pcU.fetch(chemId, returnType="view", storeRawResponsePath=rawResponsePath, storeResponsePath=extractedResponsePath)
            self.assertTrue(retStatus)
            self.assertGreaterEqual(len(vL), 1)

    # @unittest.skip("Skipping until api is more reliable")
    def testFetchCompoundExtTable(self):
        try:
            cIdList = ["2244", "123631"]
            extTable = cId = None
            for cId in cIdList:
                pcU = PubChemUtils()
                chemId = ChemicalIdentifier(idCode="test", identifierType="cid", identifier=cId)
                for extTable in ["dgidb", "pathway", "fdaorangebook", "clinicaltrials", "bioactivity"]:
                    rawResponsePath = os.path.join(self.__workPath, "%s-pubchem-%s-raw.json" % (cId, extTable))
                    extractedResponsePath = os.path.join(self.__workPath, "%s-pubchem-%s-extracted.json" % (cId, extTable))
                    retStatus, vL = pcU.fetch(chemId, returnType=extTable, storeRawResponsePath=rawResponsePath, storeResponsePath=extractedResponsePath)
                    self.assertTrue(retStatus)
                    self.assertGreaterEqual(len(vL), 1)
        except Exception as e:
            logger.exception("Failing with cId %r exTable %r %s", cId, extTable, str(e))
            self.fail()

    def testAssemble(self):
        cIdList = ["2244", "123631"]
        for cId in cIdList:
            pcU = PubChemUtils()
            chemId = ChemicalIdentifier(idCode=cId, identifierType="cid", identifier=cId)
            retStatus, retDL = pcU.assemble(chemId, exportPath=os.path.join(self.__workPath, "PubChem"))
            self.assertTrue(retStatus)
            self.assertTrue("record" in retDL[0]["data"])
            self.assertTrue("dgidb" in retDL[0]["data"])


def fetchPubChemData():
    suiteSelect = unittest.TestSuite()
    suiteSelect.addTest(PubChemUtilsTests("testFetchCompoundRecord"))
    suiteSelect.addTest(PubChemUtilsTests("testFetchCompoundView"))
    suiteSelect.addTest(PubChemUtilsTests("testFetchCompoundAltReturnTypes"))
    suiteSelect.addTest(PubChemUtilsTests("testFetchCompoundExtTable"))
    suiteSelect.addTest(PubChemUtilsTests("testSearchCompoundRecord"))
    return suiteSelect


if __name__ == "__main__":

    mySuite = fetchPubChemData()
    unittest.TextTestRunner(verbosity=2).run(mySuite)



##
# File:    DrugCentralProviderTests.py
# Author:  J. Westbrook
# Date:    11-Nov-2020
#
# Update:
#
#
##
"""
Tests for accessors for DrugCentral small molecule data.

"""

__docformat__ = "google en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2.0"

import logging
import os
import unittest

from rcsb.utils.chemref.DrugCentralProvider import DrugCentralProvider
from rcsb.utils.io.MarshalUtil import MarshalUtil

HERE = os.path.abspath(os.path.dirname(__file__))
TOPDIR = os.path.dirname(os.path.dirname(HERE))

logging.basicConfig(level=logging.INFO, format="%(asctime)s [%(levelname)s]-%(module)s.%(funcName)s: %(message)s")
logger = logging.getLogger()


class DrugCentralProviderTests(unittest.TestCase):
    def setUp(self):
        self.__cachePath = os.path.join(HERE, "test-output")
        self.__dataPath = os.path.join(HERE, "test-data")
        self.__mU = MarshalUtil(workPath=self.__cachePath)

    def tearDown(self):
        pass

    def testFetchDrugCentralData(self):
        try:
            dcP = DrugCentralProvider(cachePath=self.__cachePath, useCache=False)
            ok = dcP.testCache()
            self.assertTrue(ok)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()


def drugCentralDataSuite():
    suiteSelect = unittest.TestSuite()
    suiteSelect.addTest(DrugCentralProviderTests("testFetchDrugCentralData"))
    return suiteSelect


if __name__ == "__main__":
    mySuite = drugCentralDataSuite()
    unittest.TextTestRunner(verbosity=2).run(mySuite)



##
# File:    BirdProviderTests.py
# Author:  J. Westbrook
# Date:    1-Jun-2021
# Version: 0.001
#
# Update:
##
"""
Tests for utilities to read and process BIRD dictionary definitions.

"""

__docformat__ = "google en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2.0"

import logging
import os
import platform
import resource
import time
import unittest

from importlib.metadata import version as get_package_version
from rcsb.utils.chemref.BirdProvider import BirdProvider

__version__ = get_package_version("rcsb.utils.chemref")

HERE = os.path.abspath(os.path.dirname(__file__))
TOPDIR = os.path.dirname(os.path.dirname(os.path.dirname(HERE)))

logging.basicConfig(level=logging.INFO, format="%(asctime)s [%(levelname)s]-%(module)s.%(funcName)s: %(message)s")
logger = logging.getLogger()


class BirdProviderTests(unittest.TestCase):
    def setUp(self):
        self.__startTime = time.time()
        self.__dataPath = os.path.join(HERE, "test-data")
        self.__cachePath = os.path.join(HERE, "test-output")
        #
        logger.debug("Running tests on version %s", __version__)
        logger.info("Starting %s at %s", self.id(), time.strftime("%Y %m %d %H:%M:%S", time.localtime()))

    def tearDown(self):
        unitS = "MB" if platform.system() == "Darwin" else "GB"
        rusageMax = resource.getrusage(resource.RUSAGE_SELF).ru_maxrss
        logger.info("Maximum resident memory size %.4f %s", rusageMax / 10 ** 6, unitS)
        endTime = time.time()
        logger.info("Completed %s at %s (%.4f seconds)", self.id(), time.strftime("%Y %m %d %H:%M:%S", time.localtime()), endTime - self.__startTime)

    def testBuildBirdCache(self):
        """Test build Bird definition cache"""
        bP = BirdProvider(cachePath=self.__cachePath, useCache=False, molLimit=None)
        ok = bP.testCache(minCount=1000)
        self.assertTrue(ok)
        bD = bP.getBirdD()
        for ky in ["releaseStatus", "representAs", "chemCompId", "class"]:
            self.assertTrue(ky in bD["PRD_000288"])
        self.assertEqual(bP.getReleaseStatus("PRD_000288"), "REL")
        self.assertEqual(bP.getRepresentation("PRD_000288"), "single molecule")
        self.assertEqual(bP.getChemCompId("PRD_000288"), "0GJ")
        self.assertEqual(bP.getClassType("PRD_000288"), "Inhibitor")
        bP = BirdProvider(cachePath=self.__cachePath, useCache=True, molLimit=None)
        ok = bP.testCache(minCount=1000)
        self.assertTrue(ok)


def buildCacheFiles():
    suiteSelect = unittest.TestSuite()
    suiteSelect.addTest(BirdProviderTests("testBuildBirdCache"))
    return suiteSelect


if __name__ == "__main__":

    mySuite = buildCacheFiles()
    unittest.TextTestRunner(verbosity=2).run(mySuite)



##
# File:    RcsbLigandScoreProviderTests.py
# Author:  J. Westbrook
# Date:    8-Feb-2021
# Version: 0.001
#
# Update:
#
#
##
"""
Tests for utilities managing supporting data for RCSB ligand scoring.

"""

__docformat__ = "google en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2.0"

import logging
import os
import platform
import resource
import time
import unittest

from rcsb.utils.chemref.RcsbLigandScoreProvider import RcsbLigandScoreProvider

HERE = os.path.abspath(os.path.dirname(__file__))
TOPDIR = os.path.dirname(os.path.dirname(HERE))

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


class RcsbLigandScoreProviderTests(unittest.TestCase):
    skipFull = True

    def setUp(self):
        self.__cachePath = os.path.join(HERE, "test-output", "CACHE")
        #
        self.__startTime = time.time()
        logger.info("Starting %s at %s", self.id(), time.strftime("%Y %m %d %H:%M:%S", time.localtime()))

    def tearDown(self):
        unitS = "MB" if platform.system() == "Darwin" else "GB"
        rusageMax = resource.getrusage(resource.RUSAGE_SELF).ru_maxrss
        logger.info("Maximum resident memory size %.4f %s", rusageMax / 10 ** 6, unitS)
        endTime = time.time()
        logger.info("Completed %s at %s (%.4f seconds)", self.id(), time.strftime("%Y %m %d %H:%M:%S", time.localtime()), endTime - self.__startTime)

    def testFetchScoreFiles(self):
        rlscP = RcsbLigandScoreProvider(cachePath=self.__cachePath, useCache=False)
        ok = rlscP.testCache()
        self.assertTrue(ok)
        meanD, stdD, loadingD = rlscP.getParameterStatistics()
        for ky in meanD:
            logger.info("%-20s Mean %.4f stddev %.4f loading %.4f", ky, meanD[ky], stdD[ky], loadingD[ky])
        #
        fitRank = rlscP.getFitScoreRanking(-1.9)
        geoRank = rlscP.getGeometryScoreRanking(-1.5)
        self.assertGreaterEqual(fitRank, 0.90)
        self.assertGreaterEqual(geoRank, 0.90)
        logger.info("fitRank (-1.9) %.4f  geoRank (-1.5) %.4f", fitRank, geoRank)


def fetchLigandScoreFiles():
    suiteSelect = unittest.TestSuite()
    suiteSelect.addTest(RcsbLigandScoreProviderTests("testFetchScoreFiles"))
    return suiteSelect


if __name__ == "__main__":
    mySuite = fetchLigandScoreFiles()
    unittest.TextTestRunner(verbosity=2).run(mySuite)



##
# File:    ChEMBLProviderTests.py
# Author:  J. Westbrook
# Date:    9-Nov-2020
#
# Update:
#  2-Dec-2020 - jdw Add ChEMBL API tests
#
##
"""
Tests for accessors for ChEMBL small molecule data.

"""

__docformat__ = "google en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2.0"

import logging
import os
import unittest

from chembl_webresource_client.settings import Settings

from rcsb.utils.chemref.ChEMBLProvider import ChEMBLProvider
from rcsb.utils.io.MarshalUtil import MarshalUtil

HERE = os.path.abspath(os.path.dirname(__file__))
TOPDIR = os.path.dirname(os.path.dirname(HERE))

Settings.Instance().TIMEOUT = 10  # pylint: disable=no-member
Settings.Instance().MAX_LIMIT = 50  # pylint: disable=no-member
Settings.MAX_LIMIT = 50

logging.basicConfig(level=logging.INFO, format="%(asctime)s [%(levelname)s]-%(module)s.%(funcName)s: %(message)s")
logger = logging.getLogger()


class ChEMBLProviderTests(unittest.TestCase):
    def setUp(self):
        self.__cachePath = os.path.join(HERE, "test-output")
        self.__dataPath = os.path.join(HERE, "test-data")
        self.__mU = MarshalUtil(workPath=self.__cachePath)

    def tearDown(self):
        pass

    def testFetchChEMBLData(self):
        try:
            pcP = ChEMBLProvider(cachePath=self.__cachePath, useCache=False)
            ok = pcP.testCache()
            self.assertTrue(ok)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()

    def testFetchDrugData(self):
        try:
            logger.info("MAX_LIMIT %r", Settings.Instance().MAX_LIMIT)  # pylint: disable=no-member
            ctP = ChEMBLProvider(cachePath=self.__cachePath, useCache=True, addTaxonomy=False)
            ok = ctP.testCache()
            self.assertTrue(ok)
            mL = ["CHEMBL815", "CHEMBL426559", "CHEMBL548", "CHEMBL2442750", "CHEMBL1201379", "CHEMBL3039498", "CHEMBL3137332"]
            #
            oD = ctP.getDrugData(mL)
            #
            logger.info("molD %r", list(oD.keys()))
            ok = self.__mU.doExport(os.path.join(self.__cachePath, "ChEMBL-targets", "chembl-drug.json"), oD, fmt="json", indent=3)
            self.assertTrue(ok)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()

    def testFetchMoleculeData(self):
        try:
            logger.info("MAX_LIMIT %r", Settings.Instance().MAX_LIMIT)  # pylint: disable=no-member
            ctP = ChEMBLProvider(cachePath=self.__cachePath, useCache=True, addTaxonomy=False)
            ok = ctP.testCache()
            self.assertTrue(ok)
            mL = ["CHEMBL815", "CHEMBL426559", "CHEMBL548", "CHEMBL2442750", "CHEMBL1201379", "CHEMBL3039498", "CHEMBL3137332"]
            #
            oD = ctP.getMoleculeData(mL)
            #
            logger.info("molD %r", list(oD.keys()))
            ok = self.__mU.doExport(os.path.join(self.__cachePath, "ChEMBL-targets", "chembl-molecule.json"), oD, fmt="json", indent=3)
            self.assertTrue(ok)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()

    def testFetchMolelcuesByInChI(self):
        try:
            logger.info("MAX_LIMIT %r", Settings.Instance().MAX_LIMIT)  # pylint: disable=no-member
            ctP = ChEMBLProvider(cachePath=self.__cachePath, useCache=True, addTaxonomy=False)
            ok = ctP.testCache()
            self.assertTrue(ok)
            mL = ["NXFFJDQHYLNEJK-CYBMUJFWSA-N", "WWSWYXNVCBLWNZ-QIZQQNKQSA-N"]
            oD = ctP.getMoleculeByInChIKey(mL)
            #
            ok = self.__mU.doExport(os.path.join(self.__cachePath, "ChEMBL-targets", "chembl-inchikey-matches.json"), oD, fmt="json", indent=3)
            self.assertTrue(ok)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()

    def testFetchUniChemData(self):
        try:
            logger.info("MAX_LIMIT %r", Settings.Instance().MAX_LIMIT)  # pylint: disable=no-member
            ctP = ChEMBLProvider(cachePath=self.__cachePath, useCache=True, addTaxonomy=False)
            ok = ctP.testCache()
            self.assertTrue(ok)
            mL = ["NXFFJDQHYLNEJK-CYBMUJFWSA-N", "WWSWYXNVCBLWNZ-QIZQQNKQSA-N"]
            oD = ctP.getUniChemData(mL)
            #
            ok = self.__mU.doExport(os.path.join(self.__cachePath, "ChEMBL-targets", "unichem-inchikey-matches.json"), oD, fmt="json", indent=3)
            self.assertTrue(ok)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()

    def testGetUniChemSources(self):
        try:
            srL = self.__mU.doImport(os.path.join(self.__dataPath, "UC_SOURCE.tdd"), fmt="tdd", rowFormat="list")
            # logger.info("srDL %r", srL)
            uD = {}
            for sr in srL:
                if len(sr) < 12:
                    logger.info("bad sr %r", sr)
                    continue
                uD[int(sr[0])] = {"name": sr[1], "baseUrl": sr[10], "entryUrl": sr[11]}
            logger.debug("uD = %r", uD)
            ok = self.__mU.doExport(os.path.join(self.__cachePath, "unichem-source-list.json"), uD, fmt="json", indent=3)
            self.assertTrue(ok)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()


def chemblDataSuite():
    suiteSelect = unittest.TestSuite()
    suiteSelect.addTest(ChEMBLProviderTests("testFetchChEMBLData"))
    suiteSelect.addTest(ChEMBLProviderTests("testGetUniChemSources"))
    suiteSelect.addTest(ChEMBLProviderTests("testFetchUniChemData"))
    suiteSelect.addTest(ChEMBLProviderTests("testFetchMolelcuesByInChI"))
    suiteSelect.addTest(ChEMBLProviderTests("testFetchMoleculeData"))
    return suiteSelect


if __name__ == "__main__":
    mySuite = chemblDataSuite()
    unittest.TextTestRunner(verbosity=2).run(mySuite)



##
# File:    CARDProviderTests.py
# Author:  J. Westbrook
# Date:    27-Nov-2020
# Version: 0.001
#
# Update:
#
#
##
"""
Tests for utilities managing CARD molecule data.

"""

__docformat__ = "google en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2.0"

import logging
import os
import platform
import resource
import time
import unittest

from rcsb.utils.chemref.CARDProvider import CARDProvider

HERE = os.path.abspath(os.path.dirname(__file__))
TOPDIR = os.path.dirname(os.path.dirname(HERE))

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


class CARDProviderTests(unittest.TestCase):
    skipFull = True

    def setUp(self):
        self.__cachePath = os.path.join(HERE, "test-output", "CACHE")
        #
        self.__startTime = time.time()
        logger.info("Starting %s at %s", self.id(), time.strftime("%Y %m %d %H:%M:%S", time.localtime()))

    def tearDown(self):
        unitS = "MB" if platform.system() == "Darwin" else "GB"
        rusageMax = resource.getrusage(resource.RUSAGE_SELF).ru_maxrss
        logger.info("Maximum resident memory size %.4f %s", rusageMax / 10 ** 6, unitS)
        endTime = time.time()
        logger.info("Completed %s at %s (%.4f seconds)", self.id(), time.strftime("%Y %m %d %H:%M:%S", time.localtime()), endTime - self.__startTime)

    def testFetchCARDMolecules(self):
        cP = CARDProvider(cachePath=self.__cachePath, useCache=False)
        ok = cP.testCache()
        self.assertTrue(ok)


def fetchCARDMolecules():
    suiteSelect = unittest.TestSuite()
    suiteSelect.addTest(CARDProviderTests("testFetchCARDMolecules"))
    return suiteSelect


if __name__ == "__main__":
    mySuite = fetchCARDMolecules()
    unittest.TextTestRunner(verbosity=2).run(mySuite)



##
# File:    BirdProvider.py
# Author:  J. Westbrook
# Date:    1-Jun-2021
#
# Updates:
# 21-Jul-2021 jdw  Make this provider a subclass of StashableBase
##
"""
Utilities to read and serialize portions of the dictionary of PDBx/mmCIF BIRD definitions.
"""
__docformat__ = "google en"
__author__ = "John Westbrook"
__email__ = "john.westbrook@rcsb.org"
__license__ = "Apache 2.0"

import logging
import os
import time

from rcsb.utils.io.FileUtil import FileUtil
from rcsb.utils.io.MarshalUtil import MarshalUtil
from rcsb.utils.io.StashableBase import StashableBase

logger = logging.getLogger(__name__)


class BirdProvider(StashableBase):
    """Utilities to read and serialize portions of the dictionary of PDBx/mmCIF BIRD definitions."""

    def __init__(self, **kwargs):
        dirName = "bird"
        cachePath = kwargs.get("cachePath", ".")
        super(BirdProvider, self).__init__(cachePath, [dirName])

        # Default source target locators
        self.__birdUrlTarget = kwargs.get("birdUrlTarget", None)
        self.__birdUrlTarget = self.__birdUrlTarget if self.__birdUrlTarget else "http://files.wwpdb.org/pub/pdb/data/bird/prd/prd-all.cif.gz"
        #
        dirPath = os.path.join(cachePath, dirName)
        useCache = kwargs.get("useCache", True)
        molLimit = kwargs.get("molLimit", 0)
        #
        self.__mU = MarshalUtil(workPath=dirPath)
        self.__birdD = self.__reload(self.__birdUrlTarget, dirPath, useCache=useCache, molLimit=molLimit)

    def testCache(self, minCount=1):
        if self.__birdD and len(self.__birdD) >= minCount:
            logger.info("BIRD definitions (%d)", len(self.__birdD))
            return True
        return False

    def getBirdD(self):
        return self.__birdD

    def getRepresentation(self, prdId):
        try:
            return self.__birdD[prdId]["representAs"]
        except Exception as e:
            logger.debug("Get representation %r failing with %s", prdId, str(e))
        return None

    def getReleaseStatus(self, prdId):
        try:
            return self.__birdD[prdId]["releaseStatus"]
        except Exception as e:
            logger.debug("Get status %r failing with %s", prdId, str(e))
        return None

    def getChemCompId(self, prdId):
        try:
            return self.__birdD[prdId]["chemCompId"]
        except Exception as e:
            logger.debug("Get compId %r failing with %s", prdId, str(e))
        return None

    def getClassType(self, prdId):
        try:
            return self.__birdD[prdId]["class"]
        except Exception as e:
            logger.debug("Get class %r failing with %s", prdId, str(e))
        return None

    def __reload(self, birdUrlTarget, dirPath, useCache=False, molLimit=None):
        """Reload or create serialized data dictionary of BIRD molecule definitions.

        Args:
            birdUrlTarget (str): target url for bird dictionary resource file
            dirPath (str): path to the directory containing cache files
            useCache (bool):
            molLimit (int): maximum number of definitions to process

         Returns:
            (list): list of bird data containers
        """
        #
        birdObjD = {}
        startTime = time.time()
        birdDataFilePath = os.path.join(dirPath, "bird-definitions.json")
        _, fExt = os.path.splitext(birdDataFilePath)
        birdDataFormat = "json" if fExt == ".json" else "pickle"
        #
        if useCache and self.__mU.exists(birdDataFilePath):
            birdObjD = self.__mU.doImport(birdDataFilePath, fmt=birdDataFormat)
        elif not useCache:
            # Source component data files ...
            birdFilePath = self.__fetchUrl(birdUrlTarget, dirPath, useCache=useCache)
            birdObjD = self.__readBirdDefinitions(birdFilePath, molLimit=molLimit)
            ok = self.__mU.doExport(birdDataFilePath, birdObjD, fmt=birdDataFormat)
            logger.info("Storing %d definitions (status=%r) path: %s ", len(birdObjD), ok, birdDataFilePath)
        #
        endTime = time.time()
        logger.info("Loaded/reloaded %d definitions (%.4f seconds)", len(birdObjD), endTime - startTime)
        return birdObjD

    def __fetchUrl(self, urlTarget, dirPath, useCache=False):
        fU = FileUtil()
        fn = fU.getFileName(urlTarget)
        filePath = os.path.join(dirPath, fn)
        if not (useCache and fU.exists(filePath)):
            startTime = time.time()
            ok2 = fU.get(urlTarget, filePath)
            endTime = time.time()
            if ok2:
                logger.info("Fetched %s for resource file %s (status = %r) (%.4f seconds)", urlTarget, filePath, ok2, endTime - startTime)
            else:
                logger.error("Failing fetch for %s for resource file %s (status = %r) (%.4f seconds)", urlTarget, filePath, ok2, endTime - startTime)
        #
        return filePath

    def __readBirdDefinitions(self, birdFilePath, molLimit=None):
        birdD = {}
        try:
            startTime = time.time()
            logger.info("Reading definitions from %s", birdFilePath)
            dataContainerL = self.__mU.doImport(birdFilePath, fmt="mmcif")
            endTime = time.time()
            logger.info("Read %s with %d BIRD definitions (%.4f seconds)", birdFilePath, len(dataContainerL), endTime - startTime)
            # -------
            startTime = time.time()
            dataContainerL = dataContainerL[:molLimit] if molLimit else dataContainerL
            for dataContainer in dataContainerL:
                prdId = prdReleaseStatus = representAs = chemCompId = classType = None
                if dataContainer.exists("pdbx_reference_molecule"):
                    prdObj = dataContainer.getObj("pdbx_reference_molecule")
                    prdId = prdObj.getValueOrDefault("prd_id", 0, defaultValue=None)
                    prdReleaseStatus = prdObj.getValueOrDefault("release_status", 0, defaultValue=None)
                    representAs = prdObj.getValueOrDefault("represent_as", 0, defaultValue=None)
                    chemCompId = prdObj.getValueOrDefault("chem_comp_id", 0, defaultValue=None)
                    classType = prdObj.getValueOrDefault("class", 0, defaultValue=None)
                if prdId:
                    birdD[prdId] = {"releaseStatus": prdReleaseStatus, "representAs": representAs, "chemCompId": chemCompId, "class": classType}
            endTime = time.time()
            logger.info("Processed %d definitions (%.4f seconds)", len(birdD), endTime - startTime)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
        #
        return birdD



##
# File:    DrugBankProviderTests.py
# Author:  J. Westbrook
# Date:    17-Oct-2018
# Version: 0.001
#
# Update:
#
#
##
"""
Tests for utilities extracting data from DrugBank repository exported data files.

"""

__docformat__ = "google en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2.0"

import logging
import os
import platform
import resource
import time
import unittest

from rcsb.utils.chemref.DrugBankProvider import DrugBankProvider
from rcsb.utils.config.ConfigUtil import ConfigUtil

HERE = os.path.abspath(os.path.dirname(__file__))
TOPDIR = os.path.dirname(os.path.dirname(HERE))

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


class DrugBankProviderTests(unittest.TestCase):
    def setUp(self):
        configPath = os.path.join(HERE, "test-data", "drugbank-config-example.yml")
        configName = "site_info_configuration"
        cfgOb = ConfigUtil(configPath=configPath, defaultSectionName=configName)
        self.__user = cfgOb.get("_DRUGBANK_AUTH_USERNAME", sectionName=configName)
        self.__pw = cfgOb.get("_DRUGBANK_AUTH_PASSWORD", sectionName=configName)
        self.__cachePath = os.path.join(HERE, "test-output")
        #
        self.__startTime = time.time()
        logger.info("Starting %s at %s", self.id(), time.strftime("%Y %m %d %H:%M:%S", time.localtime()))

    def tearDown(self):
        unitS = "MB" if platform.system() == "Darwin" else "GB"
        rusageMax = resource.getrusage(resource.RUSAGE_SELF).ru_maxrss
        logger.info("Maximum resident memory size %.4f %s", rusageMax / 10 ** 6, unitS)
        endTime = time.time()
        logger.info("Completed %s at %s (%.4f seconds)", self.id(), time.strftime("%Y %m %d %H:%M:%S", time.localtime()), endTime - self.__startTime)

    def testReadAbbrevDrugBankInfo(self):
        urlTarget = os.path.join(HERE, "test-data", "full_database.zip")
        logger.info("Loading abbreviated Drugbank file %s", urlTarget)
        dbu = DrugBankProvider(urlTarget=urlTarget, cachePath=self.__cachePath, useCache=False, username=self.__user, password=self.__pw)
        dbMapD = dbu.getMapping()
        version = dbu.getVersion()
        logger.info("Drugbank %r mapping length %d", version, len(dbMapD))
        self.assertGreaterEqual(len(dbMapD["id_map"]), 340)
        dbDocL = dbu.getDocuments()
        self.assertGreaterEqual(len(dbDocL), 340)

    def testReadAbbrevDrugBankFeature(self):
        urlTarget = os.path.join(HERE, "test-data", "full_database.zip")
        logger.info("Loading abbreviated Drugbank file %s", urlTarget)
        dbu = DrugBankProvider(urlTarget=urlTarget, cachePath=self.__cachePath, useCache=False, username=self.__user, password=self.__pw)
        dbId = "DB00114"
        tS = dbu.getFeature(dbId, "smiles")
        logger.debug("%s SMILES is %r", dbId, tS)
        self.assertEqual(tS, "CC1=NC=C(COP(O)(O)=O)C(C=O)=C1O")
        fnL = ["name", "description", "indication", "pharmacodynamics", "mechanism-of-action"]
        for fn in fnL:
            fS = dbu.getFeature("DB00114", fn)
            logger.debug("%s %s is %r", dbId, fn, fS)
        # logger.info("%r %s", dbId, pprint.pformat(dbu.getD(dbId)))

    @unittest.skip("Long test")
    def testReadDrugBankInfo(self):
        dbu = DrugBankProvider(cachePath=self.__cachePath, useCache=False, username=self.__user, password=self.__pw)
        dbMapD = dbu.getMapping()
        version = dbu.getVersion()
        logger.info("Drugbank %r mapping length %d", version, len(dbMapD))
        self.assertGreaterEqual(len(dbMapD["id_map"]), 5850)
        dbDocL = dbu.getDocuments()
        self.assertGreaterEqual(len(dbDocL), 5850)

    @unittest.skip("Long test")
    def testReReadDrugBankInfo(self):
        dbu = DrugBankProvider(cachePath=self.__cachePath, useCache=False, username=self.__user, password=self.__pw)
        dbMapD = dbu.getMapping()
        logger.info("Mapping length %d", len(dbMapD))
        self.assertGreaterEqual(len(dbMapD["id_map"]), 5850)
        dbDocL = dbu.getDocuments()
        self.assertGreaterEqual(len(dbDocL), 5850)
        atcD = dbMapD["db_atc_map"]
        logger.info("atcD length %d", len(atcD))
        self.assertGreaterEqual(len(atcD), 3100)
        logger.info("atcD %r", atcD)
        #
        dbu = DrugBankProvider(cachePath=self.__cachePath, useCache=True)
        dbMapD = dbu.getMapping()
        logger.info("Mapping length %d", len(dbMapD))
        self.assertGreaterEqual(len(dbMapD["id_map"]), 5850)
        dbDocL = dbu.getDocuments()
        self.assertGreaterEqual(len(dbDocL), 5850)


def readDrugBankInfo():
    suiteSelect = unittest.TestSuite()
    suiteSelect.addTest(DrugBankProviderTests("testReadDrugBankInfo"))
    suiteSelect.addTest(DrugBankProviderTests("testReReadDrugBankInfo"))
    return suiteSelect


if __name__ == "__main__":

    mySuite = readDrugBankInfo()
    unittest.TextTestRunner(verbosity=2).run(mySuite)



##
# File:    PsiModProviderTests.py
# Author:  J. Westbrook
# Date:    10-Dec-2019
# Version: 0.001
#
# Update:
#
#
##
"""
Tests for various utilities for extracting data from PSI-MOD obo export files

"""

__docformat__ = "google en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2.0"

import logging
import os
import unittest

from rcsb.utils.chemref.PsiModProvider import PsiModProvider

HERE = os.path.abspath(os.path.dirname(__file__))
TOPDIR = os.path.dirname(os.path.dirname(HERE))

logging.basicConfig(level=logging.INFO, format="%(asctime)s [%(levelname)s]-%(module)s.%(funcName)s: %(message)s")
logger = logging.getLogger()


class PsiModProviderTests(unittest.TestCase):
    def setUp(self):
        self.__workPath = os.path.join(HERE, "test-output")

    def tearDown(self):
        pass

    def testReloadPsiMod(self):
        """Test load from source"""
        try:
            psimodP = PsiModProvider(cachePath=self.__workPath, useCache=False)
            ok = psimodP.testCache()
            self.assertTrue(ok)
            version = psimodP.getVersion()
            self.assertTrue(version is not None)
            #
            rL = psimodP.getRootNodes()
            logger.info("root nodes %d %r", len(rL), rL)
            self.assertEqual(len(rL), 1)
            #
            modId = "MOD:00000"
            nL = psimodP.getSuccessors(modId)
            self.assertEqual(len(nL), 0)
            nL = psimodP.getPredecessors(modId)
            self.assertIsNotNone(nL)
            logger.info("root children %r", nL)
            ok = psimodP.exists(modId)
            self.assertTrue(ok)
            nm = psimodP.getName(modId)
            logger.info("name %r", nm)
            self.assertEqual(nm, "protein modification")
            #
            doTree = True
            if doTree:
                modIdL = [("MOD:00032", 1), ("MOD:00538", 1), ("MOD:01156", 1), ("MOD:01157", 1)]
                for modId, numParents in modIdL:
                    nm = psimodP.getName(modId)
                    self.assertIsNotNone(nm)
                    nL = psimodP.getAdjacentParents(modId)
                    self.assertIsNotNone(nL)
                    nL = psimodP.getPredecessors(modId)
                    self.assertIsNotNone(nL)
                    nL = psimodP.getSuccessors(modId)
                    self.assertIsNotNone(nL)
                    linL = psimodP.getDescendants(modId, includeSelf=True)
                    self.assertEqual(len(linL), numParents + 1)
                    logger.debug("%a Lineage(%d) %r", modId, len(linL), linL)

                trL = psimodP.exportTreeNodeList([tup[0] for tup in modIdL])
                logger.debug("trL %r", trL)
                logger.info("Length of tree node list %d", len(trL))
                self.assertGreaterEqual(len(trL), 5)

        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()


def readPsiMod():
    suiteSelect = unittest.TestSuite()
    suiteSelect.addTest(PsiModProviderTests("testReloadPsiMod"))
    return suiteSelect


if __name__ == "__main__":
    mySuite = readPsiMod()
    unittest.TextTestRunner(verbosity=2).run(mySuite)



##
#  File:           PharosProvider.py
#  Date:           30-Jul-2020 jdw
#
#  Updated:
# 21-Jul-2021 jdw  Make this provider a subclass of StashableBase
##
"""
Accessors for Pharos ChEMBL compound assignments. (JDW: this class does not build assignments)

"""

import logging
import os.path
import time

from rcsb.utils.io.MarshalUtil import MarshalUtil
from rcsb.utils.io.StashableBase import StashableBase

logger = logging.getLogger(__name__)


class PharosProvider(StashableBase):
    """Accessors for Pharos compound assignments."""

    def __init__(self, **kwargs):
        dirName = "Pharos-mapping"
        cachePath = kwargs.get("cachePath", ".")
        super(PharosProvider, self).__init__(cachePath, [dirName])
        #
        self.__version = "6.11"
        cachePath = kwargs.get("cachePath", ".")
        useCache = kwargs.get("useCache", True)
        self.__dirPath = os.path.join(cachePath, dirName)
        #
        self.__mU = MarshalUtil(workPath=self.__dirPath)
        self.__phD = self.__reload(fmt="json", useCache=useCache)
        #

    def testCache(self, minCount=1):
        if minCount == 0:
            return True
        if self.__phD and minCount and ("identifiers" in self.__phD) and len(self.__phD["identifiers"]) >= minCount:
            return True
        return False

    def getIdentifiers(self):
        """Return a dictionary of related identifiers organized by Pharos/ChEMBL id.

        Returns:
            (dict): {phId: True}
        """
        try:
            return self.__phD["identifiers"] if self.__phD["identifiers"] else {}
        except Exception as e:
            logger.error("Failing with %r", str(e))
        return {}

    def __getAnnotFilePath(self, fmt="json"):
        stashBaseFileName = "pharos_mapped_annotations"
        fExt = ".json" if fmt == "json" else ".pic"
        fp = os.path.join(self.__dirPath, stashBaseFileName + fExt)
        return fp

    def load(self, chemblIdList, contentType, fmt="json", indent=0):
        """Load the input object of type contentType into the Pharos provider cache.

        Args:
            chemblIdList (list): Pharos ChEMBL ID assignments
            contentType (str): Pharos annotation content type (identifiers|...)
            fmt (str, optional): format string (json|pickle). Defaults to 'json'.

        Returns:
            (bool): True for success for False otherwise
        """
        ok = False
        try:
            if contentType in ["identifiers"]:
                phD = {chemblId: True for chemblId in chemblIdList}
                tS = time.strftime("%Y %m %d %H:%M:%S", time.localtime())
                self.__phD = {"version": self.__version, "created": tS, contentType: phD}
                #
                annotFilePath = self.__getAnnotFilePath(fmt=fmt)
                kwargs = {"indent": indent} if fmt == "json" else {}
                ok = self.__mU.doExport(annotFilePath, self.__phD, fmt=fmt, **kwargs)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
        return ok

    def reload(self):
        """Reload from the current cache file."""
        ok = False
        try:
            self.__phD = self.__reload(fmt="json", useCache=True)
            ok = self.__phD is not None
        except Exception as e:
            logger.exception("Failing with %s", str(e))
        return ok

    def __reload(self, fmt="json", useCache=True):
        annotFilePath = self.__getAnnotFilePath(fmt=fmt)
        #
        tS = time.strftime("%Y %m %d %H:%M:%S", time.localtime())
        pcD = {"version": self.__version, "created": tS, "identifiers": {}}
        logger.info("useCache %r annotFilePath %r", useCache, annotFilePath)
        if useCache and self.__mU.exists(annotFilePath):
            pcD = self.__mU.doImport(annotFilePath, fmt=fmt)
        return pcD
